@@ -1,35 +1,32 @@
+from __future__ import annotations
+
 import logging
 import math
 import random
-import warnings
-from typing import Any, Dict
+from typing import Any, Optional
 
 import pandas as pd
 
+from negmas import warnings
+
 from ..genius import GeniusNegotiator
 from ..helpers import create_loggers, instantiate
+from ..helpers.prob import ScipyDistribution
 from ..inout import load_genius_domain_from_folder
 from ..mechanisms import Mechanism
-from ..modeling import UncertainOpponentModel
-from ..negotiators import AspirationMixin
+from ..models.acceptance import UncertainOpponentModel
 from ..outcomes import Outcome
+from ..preferences import IPUtilityFunction, MappingUtilityFunction, UtilityFunction
 from ..sao import (
     AspirationNegotiator,
     LimitedOutcomesAcceptor,
     LimitedOutcomesNegotiator,
-    OnlyBestNegotiator,
     RandomNegotiator,
     SAOMechanism,
     SAOState,
+    TopFractionNegotiator,
     ToughNegotiator,
 )
-from ..utilities import (
-    IPUtilityFunction,
-    MappingUtilityFunction,
-    UtilityDistribution,
-    UtilityFunction,
-    UtilityValue,
-)
 from .baseline import DummyElicitor, FullKnowledgeElicitor
 from .expectors import BalancedExpector, MaxExpector, MinExpector
 from .pandora import FullElicitor, RandomElicitor
@@ -39,7 +36,6 @@ from .voi import (
     BaseElicitor,
     EStrategy,
     MeanExpector,
-    Optional,
     Query,
     RangeConstraint,
     VOIElicitor,
@@ -56,29 +52,27 @@ __all__ = ["SAOElicitingMechanism"]
 def uniform():
     loc = random.random()
     scale = random.random() * (1.0 - loc)
-    return UtilityDistribution(dtype="uniform", loc=loc, scale=scale)
+    return ScipyDistribution(type="uniform", loc=loc, scale=scale)
 
 
-def current_aspiration(
-    elicitor: "AspirationMixin", outcome: "Outcome", negotiation: "Mechanism"
-) -> "UtilityValue":
-    return elicitor.aspiration(negotiation.relative_time)
+def current_aspiration(elicitor, outcome: Outcome, negotiation: Mechanism) -> float:
+    return elicitor.utility_at(negotiation.relative_time)
 
 
 def create_negotiator(
-    negotiator_type, ufun, can_propose, outcomes, toughness, **kwargs
+    negotiator_type, preferences, can_propose, outcomes, toughness, **kwargs
 ):
     if negotiator_type == "limited_outcomes":
         if can_propose:
             negotiator = LimitedOutcomesNegotiator(
                 acceptable_outcomes=outcomes,
-                acceptance_probabilities=list(ufun.mapping.values()),
+                acceptance_probabilities=list(preferences.mapping.values()),
                 **kwargs,
             )
         else:
             negotiator = LimitedOutcomesAcceptor(
                 acceptable_outcomes=outcomes,
-                acceptance_probabilities=list(ufun.mapping.values()),
+                acceptance_probabilities=list(preferences.mapping.values()),
                 **kwargs,
             )
     elif negotiator_type == "random":
@@ -88,7 +82,7 @@ def create_negotiator(
     elif negotiator_type == "tough":
         negotiator = ToughNegotiator(can_propose=can_propose)
     elif negotiator_type in ("only_best", "best_only", "best"):
-        negotiator = OnlyBestNegotiator(
+        negotiator = TopFractionNegotiator(
             min_utility=None,
             top_fraction=1.0 - toughness,
             best_first=False,
@@ -122,17 +116,13 @@ def create_negotiator(
         if class_name.startswith("_"):
             class_name = class_name[1:]
         if class_name == "auto" or len(class_name) < 1:
-            negotiator = GeniusNegotiator.random_negotiator(
-                keep_issue_names=False, keep_value_names=False, can_propose=can_propose
-            )
+            negotiator = GeniusNegotiator.random_negotiator(can_propose=can_propose)
         else:
             negotiator = GeniusNegotiator(
                 java_class_name=class_name,
-                keep_value_names=False,
-                keep_issue_names=False,
                 can_propose=can_propose,
             )
-        negotiator.utility_function = ufun
+        negotiator.preferences = preferences
     else:
         raise ValueError(f"Unknown opponents type {negotiator_type}")
     return negotiator
@@ -160,112 +150,6 @@ def _end(x):
 
 
 class SAOElicitingMechanism(SAOMechanism):
-    @classmethod
-    def generate_config(
-        cls,
-        cost,
-        n_outcomes: int = None,
-        rand_ufuns=True,
-        conflict: float = None,
-        conflict_delta: float = None,
-        winwin=None,  # only if rand_ufuns is false
-        genius_folder: str = None,
-        n_steps=None,
-        time_limit=None,
-        own_utility_uncertainty=0.5,
-        own_uncertainty_variablility=0.0,
-        own_reserved_value=0.0,
-        own_base_agent="aspiration",
-        opponent_model_uncertainty=0.5,
-        opponent_model_adaptive=False,
-        opponent_proposes=True,
-        opponent_type="best_only",
-        opponent_toughness=0.9,
-        opponent_reserved_value=0.0,
-    ) -> Dict[str, Any]:
-        config = {}
-        if n_steps is None and time_limit is None and "aspiration" in opponent_type:
-            raise ValueError(
-                "Cannot use aspiration negotiators when no step limit or time limit is given"
-            )
-        if n_outcomes is None and genius_folder is None:
-            raise ValueError(
-                "Must specify a folder to run from or a number of outcomes"
-            )
-        if genius_folder is not None:
-            domain, agent_info, _ = load_genius_domain_from_folder(
-                folder_name=genius_folder,
-                force_single_issue=True,
-                keep_issue_names=False,
-                keep_value_names=False,
-                ignore_discount=True,
-                ignore_reserved=opponent_reserved_value is not None,
-            )
-            n_outcomes = domain.ami.n_outcomes
-            outcomes = domain.outcomes
-            elicitor_indx = 0 + int(random.random() <= 0.5)
-            opponent_indx = 1 - elicitor_indx
-            ufun = agent_info[elicitor_indx]["ufun"]
-            ufun.reserved_value = own_reserved_value
-            opp_utility = agent_info[opponent_indx]["ufun"]
-            opp_utility.reserved_value = opponent_reserved_value
-        else:
-            outcomes = [(_,) for _ in range(n_outcomes)]
-            if rand_ufuns:
-                ufun, opp_utility = UtilityFunction.generate_random_bilateral(
-                    outcomes=outcomes
-                )
-            else:
-                ufun, opp_utility = UtilityFunction.generate_bilateral(
-                    outcomes=outcomes,
-                    conflict_level=opponent_toughness,
-                    conflict_delta=conflict_delta,
-                    win_win=winwin,
-                )
-            ufun.reserved_value = own_reserved_value
-            domain = SAOMechanism(
-                outcomes=outcomes,
-                n_steps=n_steps,
-                time_limit=time_limit,
-                max_n_agents=2,
-                dynamic_entry=False,
-                outcome_type=tuple,
-                cache_outcomes=True,
-            )
-
-        true_utilities = list(ufun.mapping.values())
-        priors = IPUtilityFunction.from_ufun(
-            ufun,
-            uncertainty=own_utility_uncertainty,
-            variability=own_uncertainty_variablility,
-        )
-
-        outcomes = domain.ami.outcomes
-
-        opponent = create_negotiator(
-            negotiator_type=opponent_type,
-            can_propose=opponent_proposes,
-            ufun=opp_utility,
-            outcomes=outcomes,
-            toughness=opponent_toughness,
-        )
-        opponent_model = UncertainOpponentModel(
-            outcomes=outcomes,
-            uncertainty=opponent_model_uncertainty,
-            opponents=opponent,
-            adaptive=opponent_model_adaptive,
-        )
-        config["n_steps"], config["time_limit"] = n_steps, time_limit
-        config["priors"] = priors
-        config["true_utilities"] = true_utilities
-        config["elicitor_reserved_value"] = own_reserved_value
-        config["cost"] = cost
-        config["opp_utility"] = opp_utility
-        config["opponent_model"] = opponent_model
-        config["opponent"] = opponent
-        config["base_agent"] = own_base_agent
-        return config
-
     def __init__(
         self,
         priors,
@@ -289,7 +173,7 @@ class SAOElicitingMechanism(SAOMechanism):
         update_related_queries=True,
         resolution=0.1,
         cost_assuming_titration=False,
-        name: Optional[str] = None,
+        name: str | None = None,
     ):
         self.elicitation_state = {}
         initial_priors = priors
@@ -307,8 +191,7 @@ class SAOElicitingMechanism(SAOMechanism):
             max_n_agents=2,
             dynamic_entry=False,
             name=name,
-            outcome_type=tuple,
-            enable_callbacks=True,
+            extra_callbacks=True,
         )
         if elicitor_reserved_value is None:
             elicitor_reserved_value = 0.0
@@ -317,11 +200,11 @@ class SAOElicitingMechanism(SAOMechanism):
             screen_level=logging.DEBUG if screen_log else logging.ERROR,
         )
         user = User(
-            ufun=MappingUtilityFunction(
+            preferences=MappingUtilityFunction(
                 dict(zip(self.outcomes, self.U)), reserved_value=elicitor_reserved_value
             ),
             cost=cost,
-            ami=self.ami,
+            nmi=self.nmi,
         )
         if resolution is None:
             resolution = max(elicitor_reserved_value / 4, 0.025)
@@ -329,12 +212,12 @@ class SAOElicitingMechanism(SAOMechanism):
             strategy = None
         else:
             strategy = EStrategy(strategy=elicitation_strategy, resolution=resolution)
-            strategy.on_enter(ami=self.ami, ufun=initial_priors)
+            strategy.on_enter(nmi=self.nmi, preferences=initial_priors)
 
         def create_elicitor(type_, strategy=strategy, opponent_model=opponent_model):
             base_negotiator = create_negotiator(
                 negotiator_type=base_agent,
-                ufun=None,
+                preferences=None,
                 can_propose=True,
                 outcomes=outcomes,
                 toughness=toughness,
@@ -478,18 +361,122 @@ class SAOElicitingMechanism(SAOMechanism):
 
         if isinstance(opponent, GeniusNegotiator):
             if n_steps is not None and time_limit is not None:
-                self.ami.n_steps = None
+                self.nmi.n_steps = None
 
-        self.add(opponent, ufun=opp_utility)
-        self.add(elicitor, ufun=initial_priors)
+        self.add(opponent, preferences=opp_utility)
+        self.add(elicitor, preferences=initial_priors)
         if len(self.negotiators) != 2:
             raise ValueError(
                 f"I could not add the two negotiators {elicitor.__class__.__name__}, {opponent.__class__.__name__}"
             )
         self.total_time = 0.0
 
+    @classmethod
+    def generate_config(
+        cls,
+        cost,
+        n_outcomes: int = None,
+        rand_preferencess=True,
+        conflict: float = None,
+        conflict_delta: float = None,
+        winwin=None,  # only if rand_preferencess is false
+        genius_folder: str = None,
+        n_steps=None,
+        time_limit=None,
+        own_utility_uncertainty=0.5,
+        own_uncertainty_variablility=0.0,
+        own_reserved_value=0.0,
+        own_base_agent="aspiration",
+        opponent_model_uncertainty=0.5,
+        opponent_model_adaptive=False,
+        opponent_proposes=True,
+        opponent_type="best_only",
+        opponent_toughness=0.9,
+        opponent_reserved_value=0.0,
+    ) -> dict[str, Any]:
+        config = {}
+        if n_steps is None and time_limit is None and "aspiration" in opponent_type:
+            raise ValueError(
+                "Cannot use aspiration negotiators when no step limit or time limit is given"
+            )
+        if n_outcomes is None and genius_folder is None:
+            raise ValueError(
+                "Must specify a folder to run from or a number of outcomes"
+            )
+        if genius_folder is not None:
+            d = load_genius_domain_from_folder(
+                genius_folder,
+                ignore_reserved=opponent_reserved_value is not None,
+                ignore_discount=True,
+            ).to_single_issue(numeric=True)
+            domain = d.make_session(time_limit=120)
+
+            n_outcomes = domain.nmi.n_outcomes  # type: ignore
+            outcomes = domain.outcomes
+            elicitor_indx = 0 + int(random.random() <= 0.5)
+            opponent_indx = 1 - elicitor_indx
+            preferences = d.ufuns[elicitor_indx]
+            preferences.reserved_value = own_reserved_value
+            opp_utility = d.ufuns[opponent_indx]
+            opp_utility.reserved_value = opponent_reserved_value
+        else:
+            outcomes = [(_,) for _ in range(n_outcomes)]
+            if rand_preferencess:
+                preferences, opp_utility = UtilityFunction.generate_random_bilateral(
+                    outcomes=outcomes
+                )
+            else:
+                preferences, opp_utility = UtilityFunction.generate_bilateral(
+                    outcomes=outcomes,
+                    conflict_level=opponent_toughness,
+                    conflict_delta=conflict_delta,
+                    win_win=winwin,
+                )
+            preferences.reserved_value = own_reserved_value
+            domain = SAOMechanism(
+                outcomes=outcomes,
+                n_steps=n_steps,
+                time_limit=time_limit,
+                max_n_agents=2,
+                dynamic_entry=False,
+                cache_outcomes=True,
+            )
+
+        true_utilities = list(preferences.mapping.values())
+        priors = IPUtilityFunction.from_preferences(
+            preferences,
+            uncertainty=own_utility_uncertainty,
+            variability=own_uncertainty_variablility,
+        )
+
+        outcomes = domain.nmi.outcomes
+
+        opponent = create_negotiator(
+            negotiator_type=opponent_type,
+            can_propose=opponent_proposes,
+            preferences=opp_utility,
+            outcomes=outcomes,
+            toughness=opponent_toughness,
+        )
+        opponent_model = UncertainOpponentModel(
+            outcomes=outcomes,
+            uncertainty=opponent_model_uncertainty,
+            opponents=opponent,
+            adaptive=opponent_model_adaptive,
+        )
+        config["n_steps"], config["time_limit"] = n_steps, time_limit
+        config["priors"] = priors
+        config["true_utilities"] = true_utilities
+        config["elicitor_reserved_value"] = own_reserved_value
+        config["cost"] = cost
+        config["opp_utility"] = opp_utility
+        config["opponent_model"] = opponent_model
+        config["opponent"] = opponent
+        config["base_agent"] = own_base_agent
+        return config
+
     def loginfo(self, s: str) -> None:
-        """logs ami-level information
+        """logs nmi-level information
 
         Args:
             s (str): The string to log
@@ -569,9 +556,6 @@ class SAOElicitingMechanism(SAOMechanism):
     def plot(
         self,
         visible_negotiators=(0, 1),
-        plot_utils=True,
-        plot_outcomes=False,
-        utility_range=None,
         consider_costs=False,
     ):
         try:
@@ -591,9 +575,9 @@ class SAOElicitingMechanism(SAOMechanism):
                 history["relative_time"] = [_[0].relative_time for _ in self.history]
                 history["step"] = [_[0].step for _ in self.history]
                 history = history.loc[~history.offer.isnull(), :]
-                # ufuns = self._get_ufuns(consider_costs=consider_costs)
-                ufuns = self._get_ufuns()
-                elicitor_dist = self.negotiators[1].utility_function
+                # ufuns = self._get_preferencess(consider_costs=consider_costs)
+                ufuns = self._get_preferencess()
+                elicitor_dist = self.negotiators[1].ufun
                 outcomes = self.outcomes
 
                 utils = [tuple(f(o) for f in ufuns) for o in outcomes]
@@ -779,7 +763,7 @@ class SAOElicitingMechanism(SAOMechanism):
 
         if self.elicitation_state["agreed"]:
             self.elicitation_state["utils"] = [
-                a.user_ufun(self.state.agreement)
+                a.user_preferences(self.state.agreement)
                 if isinstance(a, BaseElicitor)
                 else a.ufun(self.state.agreement)
                 for a in self.negotiators
