@@ -1,13 +1,17 @@
+from __future__ import annotations
+
 import copy
 import operator
 import pprint
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import Callable, Iterable, List, Optional, Sequence, Tuple, Union
+from typing import Callable, Iterable, Sequence
 
-from ..common import AgentMechanismInterface
+from negmas.preferences.preferences import Preferences
+
+from ..common import NegotiatorMechanismInterface, Value
+from ..helpers.prob import ScipyDistribution
 from ..outcomes import Outcome
-from ..utilities import UtilityDistribution, UtilityFunction, UtilityValue
 from .common import _loc, _upper
 
 __all__ = [
@@ -30,11 +34,12 @@ class Constraint(ABC):
 
     def __init__(
         self,
-        full_range: Union[Sequence[Tuple[float, float]], Tuple[float, float]] = (
+        full_range: Sequence[tuple[float, float]]
+        | tuple[float, float] = (
             0.0,
             1.0,
         ),
-        outcomes: List[Outcome] = None,
+        outcomes: list[Outcome] = None,
     ):
         super().__init__()
         self.outcomes = outcomes
@@ -47,59 +52,55 @@ class Constraint(ABC):
 
     @abstractmethod
     def is_satisfied(
-        self, ufun: UtilityFunction, outcomes: Optional[Iterable[Outcome]] = None
+        self, preferences: Preferences, outcomes: Iterable[Outcome] | None = None
     ) -> bool:
         """
         Whether or not the constraint is satisfied.
         """
 
-    def __str__(self):
-        return pprint.pformat(self.__dict__)
+    @abstractmethod
+    def marginals(self, outcomes: Iterable[Outcome] = None) -> list[ScipyDistribution]:
+        ...
+
+    @abstractmethod
+    def marginal(self, outcome: Outcome) -> ScipyDistribution:
+        ...
 
     def __repr__(self):
         return self.__dict__.__repr__()
 
-    @abstractmethod
-    def marginals(
-        self, outcomes: Iterable[Outcome] = None
-    ) -> List[UtilityDistribution]:
-        ...
-
-    @abstractmethod
-    def marginal(self, outcome: "Outcome") -> UtilityDistribution:
-        ...
+    def __str__(self):
+        return pprint.pformat(self.__dict__)
 
 
 class MarginalNeutralConstraint(Constraint):
     """Constraints that do not affect the marginals of any outcomes. These constraints may only affect the joint
     distribution."""
 
-    def marginals(
-        self, outcomes: Iterable[Outcome] = None
-    ) -> List[UtilityDistribution]:
+    def marginals(self, outcomes: Iterable[Outcome] = None) -> list[ScipyDistribution]:
         if outcomes is None:
             outcomes = self.outcomes
         # this works only for real-valued outcomes.
         return [
-            UtilityDistribution(
-                dtype="uniform",
+            ScipyDistribution(
+                type="uniform",
                 loc=self.full_range[_][0],
                 scale=self.full_range[_][1] - self.full_range[_][0],
             )
             for _ in range(len(outcomes))
         ]
 
-    def marginal(self, outcome: "Outcome") -> UtilityDistribution:
+    def marginal(self, outcome: Outcome) -> ScipyDistribution:
         # this works only for real-valued outcomes.
         if self.outcomes is None:
-            return UtilityDistribution(
-                dtype="uniform",
+            return ScipyDistribution(
+                type="uniform",
                 loc=self.full_range[0],
                 scale=self.full_range[1] - self.full_range[0],
             )
         indx = self.index[outcome]
-        return UtilityDistribution(
-            dtype="uniform",
+        return ScipyDistribution(
+            type="uniform",
             loc=self.full_range[indx][0],
             scale=self.full_range[indx][1] - self.full_range[indx][0],
         )
@@ -110,24 +111,25 @@ class RankConstraint(MarginalNeutralConstraint):
 
     def __init__(
         self,
-        rankings: List[int],
-        full_range: Union[Sequence[Tuple[float, float]], Tuple[float, float]] = (
+        rankings: list[int],
+        full_range: Sequence[tuple[float, float]]
+        | tuple[float, float] = (
             0.0,
             1.0,
         ),
-        outcomes: List[Outcome] = None,
+        outcomes: list[Outcome] = None,
     ):
         super().__init__(full_range=full_range, outcomes=outcomes)
         self.rankings = rankings
 
     def is_satisfied(
-        self, ufun: UtilityFunction, outcomes: Optional[Iterable[Outcome]] = None
+        self, preferences: Preferences, outcomes: Iterable[Outcome] | None = None
     ) -> bool:
         if outcomes is None:
             outcomes = self.outcomes
         if outcomes is None:
             raise ValueError("No outcomes are  given in construction or to the call")
-        u = [(ufun(o), i) for i, o in enumerate(outcomes)]
+        u = [(preferences(o), i) for i, o in enumerate(outcomes)]
         ranking = sorted(u, key=lambda x: x[0])
         return ranking == self.rankings
 
@@ -137,12 +139,13 @@ class ComparisonConstraint(MarginalNeutralConstraint):
 
     def __init__(
         self,
-        op: Union[str, Callable[[UtilityValue, UtilityValue], bool]],
-        full_range: Union[Sequence[Tuple[float, float]], Tuple[float, float]] = (
+        op: str | Callable[[Value, Value], bool],
+        full_range: Sequence[tuple[float, float]]
+        | tuple[float, float] = (
             0.0,
             1.0,
         ),
-        outcomes: List[Outcome] = None,
+        outcomes: list[Outcome] = None,
     ):
         super().__init__(full_range=full_range, outcomes=outcomes)
         if outcomes is not None and len(outcomes) != 2:
@@ -166,7 +169,7 @@ class ComparisonConstraint(MarginalNeutralConstraint):
         self.op = op
 
     def is_satisfied(
-        self, ufun: UtilityFunction, outcomes: Optional[Iterable[Outcome]] = None
+        self, preferences: Preferences, outcomes: Iterable[Outcome] | None = None
     ) -> bool:
         if outcomes is None:
             outcomes = self.outcomes
@@ -176,7 +179,7 @@ class ComparisonConstraint(MarginalNeutralConstraint):
             raise ValueError(
                 f"{len(outcomes)} outcomes were given to {self.__class__.__name__}"
             )
-        u = [(ufun(o), i) for i, o in enumerate(outcomes)]
+        u = [(preferences(o), i) for i, o in enumerate(outcomes)]
         return self.op(u[0], u[1])
 
     def __str__(self):
@@ -190,12 +193,13 @@ class RangeConstraint(Constraint):
 
     def __init__(
         self,
-        rng: Tuple = (None, None),
-        full_range: Union[Sequence[Tuple[float, float]], Tuple[float, float]] = (
+        rng: tuple = (None, None),
+        full_range: Sequence[tuple[float, float]]
+        | tuple[float, float] = (
             0.0,
             1.0,
         ),
-        outcomes: List[Outcome] = None,
+        outcomes: list[Outcome] = None,
         eps=1e-5,
     ):
         super().__init__(full_range=full_range, outcomes=outcomes)
@@ -218,13 +222,13 @@ class RangeConstraint(Constraint):
             ]
 
     def is_satisfied(
-        self, ufun: UtilityFunction, outcomes: Optional[Iterable[Outcome]] = None
+        self, preferences: Preferences, outcomes: Iterable[Outcome] | None = None
     ) -> bool:
         if outcomes is None:
             outcomes = self.outcomes
         if outcomes is None:
             raise ValueError("No outcomes are  given in construction or to the call")
-        us = [ufun(o) for o in outcomes]
+        us = [preferences(o) for o in outcomes]
         mn, mx = self.range
         if mn is not None:
             for u in us:
@@ -236,32 +240,30 @@ class RangeConstraint(Constraint):
                     return False
         return True
 
-    def marginals(
-        self, outcomes: Iterable[Outcome] = None
-    ) -> List[UtilityDistribution]:
+    def marginals(self, outcomes: Iterable[Outcome] = None) -> list[ScipyDistribution]:
         if outcomes is None:
             outcomes = self.outcomes
         # this works only for real-valued outcomes.
         return [
-            UtilityDistribution(
-                dtype="uniform",
+            ScipyDistribution(
+                type="uniform",
                 loc=self.effective_range[_][0],
                 scale=self.effective_range[_][1] - self.effective_range[_][0],
             )
             for _ in range(len(outcomes))
         ]
 
-    def marginal(self, outcome: "Outcome") -> UtilityDistribution:
+    def marginal(self, outcome: Outcome) -> ScipyDistribution:
         # this works only for real-valued outcomes.
         if self.outcomes is None:
-            return UtilityDistribution(
-                dtype="uniform",
+            return ScipyDistribution(
+                type="uniform",
                 loc=self.effective_range[0],
                 scale=self.effective_range[1] - self.effective_range[0],
             )
         indx = self.index[outcome]
-        return UtilityDistribution(
-            dtype="uniform",
+        return ScipyDistribution(
+            type="uniform",
             loc=self.effective_range[indx][0],
             scale=self.effective_range[indx][1] - self.effective_range[indx][0],
         )
@@ -277,7 +279,7 @@ class RangeConstraint(Constraint):
 
 @dataclass
 class Answer:
-    outcomes: List[Outcome]
+    outcomes: list[Outcome]
     constraint: Constraint
     cost: float = 0.0
     name: str = ""
@@ -298,8 +300,8 @@ class Answer:
 
 @dataclass
 class Query:
-    answers: List[Answer]
-    probs: List[float]
+    answers: list[Answer]
+    probs: list[float]
     cost: float = 0.0
     name: str = ""
 
@@ -317,17 +319,17 @@ class Query:
 
 @dataclass
 class QResponse:
-    answer: Optional[Answer]
+    answer: Answer | None
     indx: int
     cost: float
 
 
 def possible_queries(
-    ami: AgentMechanismInterface,
-    strategy: "EStrategy",
-    user: "User",
-    outcome: "Outcome" = None,
-) -> List[Tuple[Outcome, List["UtilityDistribution"], float]]:
+    nmi: NegotiatorMechanismInterface,
+    strategy: EStrategy,
+    user: User,
+    outcome: Outcome = None,
+) -> list[tuple[Outcome, list[ScipyDistribution], float]]:
     """Gets all queries that could be asked for that outcome until an exact value of ufun is found.
 
     For each ask,  the following tuple is returned:
@@ -337,7 +339,7 @@ def possible_queries(
     user = copy.deepcopy(user)
     strategy = copy.deepcopy(strategy)
 
-    def _possible_queries(outcome, strategy=strategy, ami=ami):
+    def _possible_queries(outcome, strategy=strategy, nmi=nmi):
         queries_before = user.elicited_queries()
         utility_before = strategy.utility_estimate(outcome)
         lower_before, upper_before = _loc(utility_before), _upper(utility_before)
@@ -362,17 +364,17 @@ def possible_queries(
         #         continue
         #     others = []
         #     if (_[1] - lower_before) > epsilon:
-        #         others.append(UtilityDistribution(dtype='uniform', loc=lower_before, scale=_[1] - lower_before))
-        #     others.append(UtilityDistribution(dtype='uniform', loc=_[1], scale=_[2]) if _[2] > 0 else _[1])
+        #         others.append(ScipyDistribution(type='uniform', loc=lower_before, scale=_[1] - lower_before))
+        #     others.append(ScipyDistribution(type='uniform', loc=_[1], scale=_[2]) if _[2] > 0 else _[1])
         #     end = (_[1] + _[2])
         #     if (upper_before - end) > epsilon:
-        #         others.append(UtilityDistribution(dtype='uniform', loc=end, scale=upper_before - end))
+        #         others.append(ScipyDistribution(type='uniform', loc=end, scale=upper_before - end))
         #     qs[old_indx] = (_[0], others, _[3], _[4], _[5])
         return qs
 
     if outcome is None:
         queries = []
-        for outcome in ami.outcomes:
+        for outcome in nmi.outcomes:
             queries += _possible_queries(outcome)
     else:
         queries = _possible_queries(outcome)
@@ -380,8 +382,8 @@ def possible_queries(
 
 
 def next_query(
-    strategy: "EStrategy", user: "User", outcome: "Outcome" = None
-) -> List[Tuple[Outcome, Query, float]]:
+    strategy: EStrategy, user: User, outcome: Outcome = None
+) -> list[tuple[Outcome, Query, float]]:
     """Gets the possible outcomes for the next ask with its cost.
 
     The following tuple is returned:
