@@ -1,11 +1,16 @@
-import warnings
+from __future__ import annotations
+
 from dataclasses import dataclass
-from typing import List, Optional
+from typing import List
 
 import numpy as np
 
+from negmas import warnings
+from negmas.preferences.preferences import Preferences
+from negmas.types.rational import Rational
+
 from ..outcomes import Outcome
-from ..utilities import MappingUtilityFunction, UtilityFunction
+from ..preferences import MappingUtilityFunction, UtilityFunction
 from .queries import Constraint, CostEvaluator, QResponse, Query
 
 np.seterr(all="raise")  # setting numpy to raise exceptions in case of errors
@@ -18,7 +23,7 @@ class ElicitationRecord:
     cost: float
     query: Query
     answer_index: int
-    step: Optional[int] = None
+    step: int | None = None
 
     def __str__(self):
         if self.step is None:
@@ -28,38 +33,36 @@ class ElicitationRecord:
     __repr__ = __str__
 
 
-class User:
+class User(Rational):
     """Abstract base class for all representations of users used for elicitation
 
     Args:
-        ufun: The real utility function of the user.
+        preferences: The real utility function of the user (pass either ufun or preferences).
+        ufun: The real utility function of the user (pass either ufun or preferences).
         cost: A cost to be added for every question asked to the user.
-        ami: [Optional] The `AgentMechanismInterface` representing *the*
+        nmi: [Optional] The `AgentMechanismInterface` representing *the*
              negotiation session engaged in by this user using this `ufun`.
 
     """
 
-    def __init__(
-        self, ufun: Optional[UtilityFunction] = None, cost: float = 0.0, ami=None
-    ):
-        super().__init__()
-        self.utility_function = ufun
+    def __init__(self, cost: float = 0.0, nmi=None, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.cost = cost
         self.total_cost = 0.0
-        self._elicited_queries: List[ElicitationRecord] = []
-        self.ami = ami
+        self._elicited_queries: list[ElicitationRecord] = []
+        self.nmi = nmi
 
-    def set(self, ufun: Optional[UtilityFunction] = None, cost: float = None):
+    def set(self, preferences: Preferences | None = None, cost: float = None):
         """
         Sets the ufun and/or cost for this user        <`0:desc`>
 
         Args:
-            ufun: The ufun if given
+            preferences: The ufun if given
             cost: The cost if given
         """
 
-        if ufun is not None:
-            self.utility_function = ufun
+        if preferences is not None:
+            self._preferences = preferences
         if cost is not None:
             self.cost = cost
 
@@ -67,12 +70,12 @@ class User:
     def ufun(self) -> UtilityFunction:
         """Gets a `UtilityFunction` representing the real utility_function of the user"""
         return (
-            self.utility_function
-            if self.utility_function is not None
+            self._preferences
+            if self._preferences is not None
             else MappingUtilityFunction(lambda x: None)
         )
 
-    def ask(self, q: Optional[Query]) -> QResponse:
+    def ask(self, q: Query | None) -> QResponse:
         """Query the user and get a response."""
         if q is None:
             return QResponse(answer=None, indx=-1, cost=0.0)
@@ -85,17 +88,19 @@ class User:
                         query=q,
                         cost=self.cost + q.cost + reply.cost,
                         answer_index=i,
-                        step=self.ami.state.step if self.ami else None,
+                        step=self.nmi.state.step if self.nmi else None,
                     )
                 )
                 return QResponse(
                     answer=reply, indx=i, cost=CostEvaluator(self.cost)(q, reply)
                 )
-        warnings.warn(f"No response for {q} (ufun={self.ufun})")
+        warnings.warn(
+            f"No response for {q} (ufun={self.ufun})", warnings.NegmasNoResponseWarning
+        )
         return QResponse(answer=None, indx=-1, cost=q.cost)
 
     def cost_of_asking(
-        self, q: Optional[Query] = None, answer_id: int = -1, estimate_answer_cost=True
+        self, q: Query | None = None, answer_id: int = -1, estimate_answer_cost=True
     ) -> float:
         """
         Returns the cost of asking the given `Quers`.
@@ -127,7 +132,7 @@ class User:
         """
         return constraint.is_satisfied(self.ufun, outcomes=outcomes)
 
-    def elicited_queries(self) -> List[ElicitationRecord]:
+    def elicited_queries(self) -> list[ElicitationRecord]:
         """Returns a list of elicited queries.
 
         For each elicited query, the following dataclass is returned:
