@@ -12,6 +12,7 @@ from .strings import *
 from .logging import *
 from .timeout import *
 from .misc import *
+from .numba_checks import *
 
 __all__ = (
     types.__all__
@@ -19,6 +20,7 @@ __all__ = (
     + logging.__all__
     + timeout.__all__
     + misc.__all__
+    + numba_checks.__all__
     # + ["prob", "inout", "numeric"]
     # + numeric.__all__
     # + prob.__all__
