@@ -6,6 +6,7 @@ from __future__ import annotations
 import xml.etree.ElementTree as ET
 from os import PathLike, listdir
 from pathlib import Path
+from random import shuffle
 from typing import Any, Callable, Iterable, Sequence
 
 from attr import define
@@ -23,13 +24,12 @@ from .preferences import (
     UtilityFunction,
     conflict_level,
     make_discounted_ufun,
-    nash_point,
+    nash_points,
     opposition_level,
     pareto_frontier,
     winwin_level,
 )
 from .preferences.value_fun import TableFun
-from .sao import SAOMechanism
 
 __all__ = [
     "Scenario",
@@ -106,20 +106,30 @@ class Scenario:
         """
         raise NotImplementedError()
 
-    def to_single_issue(self, numeric=False, stringify=True) -> Scenario:
+    def _randomize(self) -> Scenario:
+        """
+        Randomizes the outcomes in a single issue scneario
+        """
+        shuffle(self.agenda.issues[0].values)
+        return self
+
+    def to_single_issue(
+        self, numeric=False, stringify=True, randomize=False
+    ) -> Scenario:
         """
         Forces the domain to have a single issue with all possible outcomes
 
         Args:
             numeric: If given, the output issue will be a `ContiguousIssue` otherwise it will be a `DiscreteCategoricalIssue`
             stringify:  If given, the output issue will have string values. Checked only if `numeric` is `False`
+            randomize: Randomize outcome order when creating the single issue
 
         Remarks:
             - maps the agenda and ufuns to work correctly together
             - Only works if the outcome space is finite
         """
         if hasattr(self.agenda, "issues") and len(self.agenda.issues) == 1:
-            return self
+            return self if not randomize else self._randomize()
         outcomes = list(self.agenda.enumerate_or_sample())
         sos = self.agenda.to_single_issue(numeric, stringify)
         ufuns = []
@@ -131,7 +141,7 @@ class Scenario:
                 while isinstance(v, DiscountedUtilityFunction):
                     u, v = v, v.ufun
                 u.ufun = LinearAdditiveUtilityFunction(
-                    values=(TableFun(dict(zip(souts, [v(_) for _ in outcomes]))),),
+                    values=(TableFun(dict(zip(souts, [v(_) for _ in outcomes]))),),  # type: ignore (The error comes from having LRU cach for table ufun's minmax which should be OK)
                     bias=0.0,
                     reserved_value=v.reserved_value,
                     name=v.name,
@@ -141,7 +151,7 @@ class Scenario:
                 continue
             ufuns.append(
                 LinearAdditiveUtilityFunction(
-                    values=(TableFun(dict(zip(souts, [u(_) for _ in outcomes]))),),
+                    values=(TableFun(dict(zip(souts, [u(_) for _ in outcomes]))),),  # type: ignore (The error comes from having LRU cach for table ufun's minmax which should be OK)
                     bias=0.0,
                     reserved_value=u.reserved_value,
                     name=u.name,
@@ -150,12 +160,15 @@ class Scenario:
             )
         self.ufuns = tuple(ufuns)
         self.agenda = sos
-        return self
+        return self if not randomize else self._randomize()
 
     def make_session(
         self,
-        negotiators: Callable[[], Negotiator] | list[Negotiator] | None = None,
-        n_steps: int | None = None,
+        negotiators: Callable[[], Negotiator]
+        | list[Negotiator]
+        | tuple[Negotiator]
+        | None = None,
+        n_steps: int | float | None = None,
         time_limit: float | None = None,
         roles: list[str] | None = None,
         **kwargs,
@@ -275,8 +288,9 @@ class Scenario:
         frontier_outcomes = tuple(
             outcomes[_] for _ in frontier_indices if _ is not None
         )
-        nash_utils, nash_indx = nash_point(ufuns, frontier_utils, outcome_space=agenda)
-        nash_outcome = outcomes[nash_indx] if nash_indx else None
+        pts = nash_points(ufuns, frontier_utils, outcome_space=agenda)
+        nash_utils, nash_indx = pts[0] if pts else (None, None)
+        nash_outcome = frontier_outcomes[nash_indx] if nash_indx else None
         minmax = [u.minmax() for u in ufuns]
         nu, no, ol, cl, wl, fu, fo = (
             dict(),
@@ -303,13 +317,13 @@ class Scenario:
                 fu_, findx = pareto_frontier(
                     us, outcomes=outcomes, sort_by_welfare=True
                 )
+                foutcomes_ = [outcomes[i] for i in findx]
                 fu[(u1.name, u2.name)], fo[(u1.name, u2.name)] = fu_, [
                     outcomes[_] for _ in findx
                 ]
-                nu[(u1.name, u2.name)], nindx = nash_point(
-                    (u1, u2), fu_, outcomes=outcomes
-                )
-                no[(u1.name, u2.name)] = outcomes[nindx] if nindx else None
+                pts = nash_points((u1, u2), fu_, outcomes=outcomes)
+                nu[(u1.name, u2.name)], nindx = pts[0] if pts else (None, None)
+                no[(u1.name, u2.name)] = foutcomes_[nindx] if nindx else None
                 ol[(u1.name, u2.name)] = opposition_level(
                     (u1, u2),
                     outcomes=outcomes,
@@ -513,6 +527,7 @@ def load_genius_domain(
         A `Domain` ready to run
 
     """
+    from .sao import SAOMechanism
 
     issues = None
     if domain_file_name is not None:
@@ -544,7 +559,6 @@ def load_genius_domain(
             }
         )
     if domain_file_name is not None:
-        kwargs["avoid_ultimatum"] = False
         kwargs["dynamic_entry"] = False
         kwargs["max_n_agents"] = None
         if not ignore_discount:
