@@ -6,10 +6,9 @@ from __future__ import annotations
 
 from .common import *
 from .components import *
-from .controllers import *
 from .mechanism import *
 from .negotiators import *
-from .plots import *
+from .controllers import *
 
 __all__ = (
     common.__all__
@@ -17,5 +16,4 @@ __all__ = (
     + mechanism.__all__
     + negotiators.__all__
     + controllers.__all__
-    + plots.__all__
 )
