@@ -2,26 +2,28 @@
 Implements negotiators for the SAO mechanism.
 """
 from .base import *
-from .modular import *
 from .limited import *
 from .tough import *
-from .controlled import *
 from .utilbased import *
 from .timebased import *
 from .titfortat import *
 from .randneg import *
 from .nice import *
+from .scs import *
+from .micro import *
+from .controlled import *
 
 
 __all__ = (
     base.__all__
-    + modular.__all__
     + limited.__all__
     + tough.__all__
-    + controlled.__all__
     + utilbased.__all__
     + timebased.__all__
     + titfortat.__all__
     + randneg.__all__
     + nice.__all__
+    + scs.__all__
+    + micro.__all__
+    + controlled.__all__
 )
