@@ -1,7 +1,8 @@
 #!/usr/bin/env python
-"""The NegMAS universal command line tool"""
 from __future__ import annotations
 
+"""The NegMAS universal command line tool"""
+
 import json
 import os
 import pathlib
@@ -18,6 +19,7 @@ from rich import print
 from tabulate import tabulate
 
 import negmas
+from negmas.config import negmas_config
 from negmas.genius.common import DEFAULT_JAVA_PORT
 from negmas.helpers import humanize_time, unique_name
 from negmas.helpers.inout import load
@@ -55,7 +57,7 @@ DEFAULT_NEGOTIATOR = "negmas.sao.AspirationNegotiator"
 def default_log_path():
     """Default location for all logs"""
 
-    return Path.home() / "negmas" / "logs"
+    return Path(negmas_config("log_base", Path.home() / "negmas" / "logs"))
 
 
 def default_tournament_path():
