@@ -1,14 +1,17 @@
+from __future__ import annotations
+
 import pytest
 
-from negmas import Issue
+from negmas.outcomes import generate_issues
+from negmas.outcomes.outcome_space import CartesianOutcomeSpace
 
-from .fixtures import *
+from .fixtures import bissue, cissue, dissue, hamlet, uissue
 
 
 def test_type(hamlet, cissue, dissue):
     assert cissue.type == "continuous", "Continuous type passes"
-    assert dissue.type == "discrete", "int passes"
-    assert hamlet.type == "discrete", "string list passes"
+    assert dissue.type == "contiguous", "int passes"
+    assert hamlet.type == "categorical", "string list passes"
 
 
 def test_is_continuous(cissue, dissue, hamlet):
@@ -18,9 +21,9 @@ def test_is_continuous(cissue, dissue, hamlet):
 
 
 def test_is_discrete(cissue, dissue, hamlet):
-    assert cissue.is_countable() is False, "Continuous type passes"
-    assert dissue.is_countable() is True, "string list passes"
-    assert hamlet.is_countable() is True, "int passes"
+    assert cissue.is_discrete() is False, "Continuous type passes"
+    assert dissue.is_discrete() is True, "string list passes"
+    assert hamlet.is_discrete() is True, "int passes"
 
 
 def test_string_conversion(uissue, hamlet):
@@ -35,10 +38,10 @@ def test_cartinatlity(cissue, dissue, hamlet):
 
 
 def test_n_outcomes(cissue, dissue, hamlet):
-    assert Issue.num_outcomes([cissue, dissue, hamlet]) == float("inf")
-    assert Issue.num_outcomes([dissue, hamlet]) == 50
-    assert Issue.num_outcomes([dissue]) == 10
-    assert Issue.num_outcomes([]) == 1
+    assert CartesianOutcomeSpace([cissue, dissue, hamlet]).cardinality == float("inf")
+    assert CartesianOutcomeSpace([dissue, hamlet]).cardinality == 50
+    assert CartesianOutcomeSpace([dissue]).cardinality == 10
+    assert CartesianOutcomeSpace([]).cardinality == 1
 
 
 def test_rand(cissue, dissue, hamlet, bissue):
@@ -67,7 +70,7 @@ def test_possibilities(cissue, dissue, hamlet, bissue):
 
 def test_issue_generation_defaults():
     options = ["a", "b", "c"]
-    issues = Issue.generate([(0.0, 1.0), options, 5])
+    issues = generate_issues([(0.0, 1.0), options, 5])
     assert len(issues) == 3
     assert (
         issues[0].is_continuous()
@@ -82,7 +85,7 @@ def test_issue_generation_defaults():
 
 
 def test_issue_generation_multiples():
-    issues_ = Issue.generate([5], [10])
+    issues_ = generate_issues([5], [10])
     assert len(issues_) == 10
     for i, issue in enumerate(issues_):
         assert issue.name.startswith(str(i))
